

#define STRICT
#define ORBITER_MODULE

#include <stdio.h>
#include <windows.h>
#include <Orbitersdk.h>
#include "UmmuSDK.h"
#include "UCGOCargoSDK.h"
#include "StationUCGO.h"

DLLCLBK void InitModule(HINSTANCE hModule)
{
	g_hDLL = hModule;
}

DLLCLBK void ExitModule(HINSTANCE hModule)
{
}

DLLCLBK VESSEL *ovcInit(OBJHANDLE hvessel, int flightmodel)
{
	return new StationUCGO(hvessel, flightmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel)
{
	if (vessel) delete (StationUCGO*)vessel;
}

void StationUCGO::clbkLoadStateEx(FILEHANDLE scn, void *status)
{
	char *line;
	int AirlockState;

	while (oapiReadScenario_nextline(scn, line)) {
		if(Crew.LoadAllMembersFromOrbiterScenario(line) == TRUE)
			continue;
		if (hUcgo.LoadCargoFromScenario(line) == TRUE)
			continue;
		if (_strnicmp(line, "AIRLOCKSTATE", 12) == 0) {
			sscanf_s(line+12, "%d", &AirlockState);
			Crew.SetAirlockDoorState(AirlockState == 1);
		} else {
			ParseScenarioLineEx(line, status);
		}
	}
}

void StationUCGO::clbkSaveState(FILEHANDLE scn)
{
	// Save default state
	VESSEL3::clbkSaveState(scn);

	// Record the airlock state
	oapiWriteScenario_int(scn, "AIRLOCKSTATE", (int)Crew.GetAirlockDoorState());

	// Save all UMMU members to scenario file
	Crew.SaveAllMembersInOrbiterScenarios(scn);

	// Save UCGO cargo to scenario file
	hUcgo.SaveCargoToScenario(scn);
}

char *StationUCGO::SendUMMUHudMessage(void)
{
	dUMMUHudMessageDelay = 15;
	return cUmmuHudDisplay;
}

char *StationUCGO::SendUCGOHudMessage(void)
{
	dUCGOHudMessageDelay = 15;
	return cUCGOHudDisplay;
}

void StationUCGO::clbkSetClassCaps(FILEHANDLE cfg)
{
	// Nothing is done for thrusters, docking points, etc.  This only
	// gives UMMU and UCGO support.
	UMMU_Init(cfg);
	UCGO_Init(cfg);
}

void StationUCGO::clbkPostStep (double simtt, double simdt, double mjd)
{
	int ReturnCode = Crew.ProcessUniversalMMu();
	switch(ReturnCode)
	{
	case UMMU_TRANSFERED_TO_OUR_SHIP: 
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "%s \"%s\" aged %i was transfered to the station",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),Crew.GetLastEnteredCrewName()
		,Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	case UMMU_RETURNED_TO_OUR_SHIP:
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "%s \"%s\" aged %i entered the station",
		Crew.GetCrewMiscIdByName(Crew.GetLastEnteredCrewName()),
		Crew.GetLastEnteredCrewName(),Crew.GetCrewAgeByName(Crew.GetLastEnteredCrewName()));
		break;
	}

	int ActionAreaReturnCode = Crew.DetectActionAreaActivated();
	if(ActionAreaReturnCode == AIRLOCK_ACTION_AREA) {
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
	}

	if(GroundContact() == TRUE)
	{
		// we check vertical speed
		int I;
		VECTOR3 vHorizonAirspeedVector = {0};
		GetHorizonAirspeedVector(vHorizonAirspeedVector);
		double VertSpeed = vHorizonAirspeedVector.y;
		if(VertSpeed < -3)
		{
			// we touched ground with more than -3 m/s, sorry dude, time to kill you all :(
			for(I = 0; I < Crew.GetCrewTotalNumber(); I++)
			{
				Crew.SetCrewMemberPulseBySlotNumber(I, 0);	// set cardiac pulse to zero
			}
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Oooh no! Crash - All crew aboard killed");
		}
	}

	Crew.WarnUserUMMUNotInstalled("StationUCGO");
	hUcgo.WarnUserUCGONotInstalled("StationUCGO");
	AddUMmuToVessel();	// Add crew without scenario editor
} 

bool StationUCGO::clbkLoadGenericCockpit(void) {
	VESSEL3::SetCameraDefaultDirection(CameraVector[ActiveDockNumber]);
	// SDK docs lie.  Need to call the oapi camera bit here to enforce the newly set camera direction
	if (CameraSwitchPending == true) {
		oapiCameraSetCockpitDir(0, 0);
		CameraSwitchPending = false;
	}
	return 1;
}

// Override to allow user to specify that the default HUD should be turned off
// (makes it much easier to see the text from this module
// This particular method was added for Orbiter 2010
void StationUCGO::clbkRenderHUD(int mode, const HUDPAINTSPEC *hps, SURFHANDLE hDefaultTex)
{

	if (ShowDefaultHUD == TRUE) {
		VESSEL3::clbkRenderHUD(mode, hps, hDefaultTex);
	}

	return;
}

// default hud can be disabled by setting ShowDefaultHUD = FALSE in vessel config file
// You will always get the default 2 MFD spots.  This just enables/disables the
// HUD showing pitch,roll,yaw information.
bool StationUCGO::clbkDrawHUD(int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	if (ShowDefaultHUD == TRUE) {
		VESSEL3::clbkDrawHUD(mode, hps, skp);
	}

	VESSEL3::SetCameraDefaultDirection(CameraVector[ActiveDockNumber]);
	// SDK docs lie.  Need to call the oapi camera bit here to enforce the newly set camera direction
	if (CameraSwitchPending == true) {
		oapiCameraSetCockpitDir(0, 0);
		CameraSwitchPending = false;
	}

	// UCGO messages
	if (dUCGOHudMessageDelay > 0)
	{
		skp->Text(5, hps->H / 60 * 17, cUCGOHudDisplay, strlen(cUCGOHudDisplay));
		dUCGOHudMessageDelay -= oapiGetSimStep();
		if (dUCGOHudMessageDelay < 0)
			dUCGOHudMessageDelay = 0;
	}

	// UMMU messages
	if(dUMMUHudMessageDelay > 0)
	{
		skp->Text(5, hps->H / 60 * 20, cUmmuHudDisplay, strlen(cUmmuHudDisplay));
		dUMMUHudMessageDelay -= oapiGetSimStep();
		if(dUMMUHudMessageDelay < 0)
			dUMMUHudMessageDelay = 0;
	}

	return true;
}

int StationUCGO::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if (!down) 
		return 0; 

	// Decrement selected MMU slot.
	if(key == OAPI_KEY_1 && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		if(Crew.GetCrewTotalNumber() == 0)
		{
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Sorry no one aboard unable to select.");
			return 1;
		}
		if(SelectedUmmuMember > 0)
			SelectedUmmuMember--;
		char *Name = Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay),
				"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer.  Press \"E\" to EVA.", 
				SelectedUmmuMember,
				Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember), Name,
				Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember));
		return 1;
	}

	// Increment selected MMU slot.
	if(key == OAPI_KEY_2 && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		// we test there is someone aboard
		if(Crew.GetCrewTotalNumber() == 0)
		{
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Sorry no one aboard unable to select");	
			return 1;
		}

		// we test that we select existing member
		if(SelectedUmmuMember < Crew.GetCrewTotalNumber()-1)
			SelectedUmmuMember++;
		char *Name = Crew.GetCrewNameBySlotNumber(SelectedUmmuMember);
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay),
				"Slot %i  %s \"%s\" aged %i Selected for EVA or Transfer.  Press \"E\" to EVA.",
				SelectedUmmuMember,
				Crew.GetCrewMiscIdBySlotNumber(SelectedUmmuMember),
				Name,
				Crew.GetCrewAgeBySlotNumber(SelectedUmmuMember)
				);
		return 1;
	}

	// Decrement selected Cargo slot.
	if (key == OAPI_KEY_3 && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		if(SelectedCargoSlot > 0) {
			SelectedCargoSlot--;
			SelectedCargoSlot %= MaxCargoSlots;
		} else {
			SelectedCargoSlot = MaxCargoSlots - 1;
		}
		sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"Cargo slot %i selected: %s with mass %0.2f kg.",
					SelectedCargoSlot,
					hUcgo.GetCargoDescriptionString(SelectedCargoSlot),
					hUcgo.GetCargoSlotMass(SelectedCargoSlot)
					);
		return 1;
	}

	// Increment selected Cargo slot.
	if (key == OAPI_KEY_4 && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		SelectedCargoSlot++;
		SelectedCargoSlot %= MaxCargoSlots;
		sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"Cargo slot %i selected: %s with mass %0.2f kg.",
					SelectedCargoSlot,
					hUcgo.GetCargoDescriptionString(SelectedCargoSlot),
					hUcgo.GetCargoSlotMass(SelectedCargoSlot)
					);
		return 1;
	}

	// Show keys available for MMU actions.
	if (key == OAPI_KEY_5) {
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), CONTROL_MENU_UMMU);
			return 1;
	}

	// Show keys available for UCGO actions.
	if (key == OAPI_KEY_6) {
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), CONTROL_MENU_UCGO);
			return 1;
	}

	// Show MMU status.
	if (key == OAPI_KEY_7) {
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay),
					"%i of %i seats in use aboard station %s.  MMU %i (%s) selected.  Active EVA dock is %i.  Airlocks are %s.",
					Crew.GetCrewTotalNumber(),
					MaxUmmuSeats,
					GetName(),
					SelectedUmmuMember,
					Crew.GetCrewNameBySlotNumber(SelectedUmmuMember),
					ActiveDockNumber,
					((Crew.GetAirlockDoorState() == TRUE) ? "open" : "closed")
					);
		return 1;
	}

	// Show Cargo status.
	if(key == OAPI_KEY_8 && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate))
	{
		sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
			"%i of %i cargo slots in use.  Cargo mass: %0.2f kg of %0.2f kg.  Slot %i (%s) selected.",
			hUcgo.GetNbrCargoLoaded(),
			MaxCargoSlots,
			hUcgo.GetCargoTotalMass(),
			MaxCargoMass,
			SelectedCargoSlot,
			hUcgo.GetCargoDescriptionString(SelectedCargoSlot)
			);
		return 1;
	}

	// Select next cargo from disk.
	if (key == OAPI_KEY_9 && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"%s selected for slot %i.",
					hUcgo.ScnEditor_SelectNextCargoAvailableOnDisk(),
					SelectedCargoSlot
					);
		return 1;
	}

	// Load selected disk cargo into current cargo slot.
	if (key == OAPI_KEY_9 && KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		if (hUcgo.GetSlotLoaded(SelectedCargoSlot) == FALSE) {
			if (hUcgo.ScnEditor_AddLastSelectedCargoToSlot(SelectedCargoSlot) == TRUE) {
				sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
						"%s added to station in slot %i.",
						hUcgo.GetCargoDescriptionString(SelectedCargoSlot),
						SelectedCargoSlot
						);
			} else {
				sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
						"Cargo not added.  Cargo maximum mass of %0.2f would be exceeded.",
						MaxCargoMass
						);
			}
		} else {
			sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"Cargo not added.  Cargo slot %i already contains cargo.",
					SelectedCargoSlot
					);
		}
		return 1;
	}

	// Toggle airlock state.
	// There is only one GLOBAL airlock state no matter how many EVA points are defined.
	if(key == OAPI_KEY_A && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		Crew.SetAirlockDoorState(!Crew.GetAirlockDoorState());
		if(Crew.GetAirlockDoorState() == TRUE) {
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Airlocks opening.");
		} else {
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Airlocks closing.");
		}
		return 1;
	}

	// Grapple cargo into currently selected cargo slot.
	if(key == OAPI_KEY_C && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate))
	{
		int ReturnedCode = hUcgo.GrappleOneCargo(SelectedCargoSlot);
		switch(ReturnedCode)
		{
		case 1:
			sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"%s grappled into cargo slot %i",
					hUcgo.GetCargoDescriptionString(SelectedCargoSlot),
					SelectedCargoSlot
					);
			break;
		case 0:
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "No cargo in range.");
			break;
		case -1:
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Cargo would exceed maximum allowable cargo mass.");
			break;
		case -2:
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Bad config, mesh not found or slot not declared.");
			break;
		case -3:
			sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"Can't grapple cargo, slot %i is not empty.",
					SelectedCargoSlot
					);
			break;
		case -4:
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Can't grapple cargo, cargo doors are closed.");
			break;
		case -5:
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Can't grapple cargo, station is full.");
			break;
		default:
			strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Misc error.  Unable to grapple cargo.");
		}
		return 1;
	}

	// Release cargo from currently selected cargo slot.
	if(key == OAPI_KEY_C && KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate))
	{
		if (hUcgo.GetSlotLoaded(SelectedCargoSlot) == TRUE) {
			char *Name = hUcgo.GetCargoDescriptionString(SelectedCargoSlot);
			if(hUcgo.ReleaseOneCargo(SelectedCargoSlot) != FALSE) {
				sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
						"%s released from cargo slot %i",
						Name,
						SelectedCargoSlot
						);
			} else {
				strcpy_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Misc error.  Unable to release cargo.");
			}
		} else {
			sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay),
					"Unable to release cargo.  Slot %i is empty.",
					SelectedCargoSlot
					);
		}
		return 1;
	}

	// EVA or transfer currently selected MMU.
	if(key == OAPI_KEY_E && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		int Returned=Crew.EvaCrewMember(Crew.GetCrewNameBySlotNumber(SelectedUmmuMember));
		switch(Returned)
		{
		case TRANSFER_TO_DOCKED_SHIP_OK:
			sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Transfer to docked ship Ok - %s transfered.",
				Crew.GetLastEvaedCrewName());
			SelectedUmmuMember = 0;
			break;
		case EVA_OK:
			sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "EVA OK - %s left the station.",
				Crew.GetLastEvaedCrewName());
			SelectedUmmuMember = 0;
			break;
		case ERROR_NO_ONE_ON_BOARD:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Error, no one on board, unable to EVA.");
			break;
		case ERROR_AIRLOCK_CLOSED:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Error, airlocks are closed, unable to EVA.");
			break;
		case ERROR_DOCKED_SHIP_HAVE_AIRLOCK_CLOSED:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Error, docked ship's airlocks are closed, unable to transfer.");
			break;
		case ERROR_DOCKED_SHIP_IS_FULL:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Error, docked ship is already full transfer failed.");
			break;
		case ERROR_CREW_MEMBER_NOT_FOUND:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Error, no crew by this name on the station.");
			break;
		case ERROR_DOCKEDSHIP_DONOT_USE_UMMU:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Error, docked ship does not use UMmu 2.0, ask author to add it.");
			break;
		case ERROR_MISC_ERROR_EVAFAILED:
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Misc error with UMMU install it again.");
			break;
		}
		return TRUE;
	}

	// Add MMU to the station without scenario editor.
	if (key == OAPI_KEY_M && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		AddUMmuToVessel(TRUE);
	}

	// Decrement active dock
	if (key == OAPI_KEY_MINUS && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		if (ActiveDockNumber > 0)
			ActiveDockNumber--;
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay),
			"Active dock number changed to %i", ActiveDockNumber);
		SetUMMUAirlockPos();
		CameraSwitchPending = true;
		return 1;
	}

	// Increment active dock
	if (key == OAPI_KEY_EQUALS && !KEYMOD_SHIFT(kstate) && !KEYMOD_CONTROL(kstate)) {
		if (ActiveDockNumber < MaxEVASpots - 1)
			ActiveDockNumber++;
		sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay),
			"Active dock number changed to %i", ActiveDockNumber);
		SetUMMUAirlockPos();
		CameraSwitchPending = true;
		return 1;
	}
	
	return 0;
}

bool UMmuCrewAddCallback(void *id, char *str, void *data)
{
	if(strlen(str)<2||strlen(str)>38)
		return false;
	char *cPtr = (char*)data;
	if (*cPtr == 2) {*cPtr=3;strcpy_s(cPtr+2,38,str);}
	else if(*cPtr==4){*cPtr=5;strcpy_s(cPtr+42,38,str);}
	else if(*cPtr==6){*cPtr=7;strcpy_s(cPtr+82,38,str);}return true;
}

void StationUCGO::AddUMmuToVessel(BOOL bStartAdding)
{
	if(bStartAdding == FALSE && cAddUMmuToVessel[0] == 0)
		return;
	if(bStartAdding == TRUE){
		int salut = sizeof(cAddUMmuToVessel);
		memset(cAddUMmuToVessel, 0, sizeof(cAddUMmuToVessel));
		cAddUMmuToVessel[0] = 1;
	}
	else if(cAddUMmuToVessel[0] == 1){
		cAddUMmuToVessel[0] = 2;
		oapiOpenInputBox ("Enter new crew's name (or escape)", UMmuCrewAddCallback, 0, 30, (void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0] == 3){
		cAddUMmuToVessel[0] = 4;
		oapiOpenInputBox ("Enter crew's age", UMmuCrewAddCallback, 0, 30, (void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0] == 5){
		cAddUMmuToVessel[0] = 6;
		oapiOpenInputBox ("Enter function (Capt,Sec,Vip,Sci,Doc,Tech,Crew,Pax)", UMmuCrewAddCallback, 0, 30, (void*)cAddUMmuToVessel);
	}
	else if(cAddUMmuToVessel[0] == 7){
		cAddUMmuToVessel[0] = 0;
		int Age = max(5, min(100, atoi(&cAddUMmuToVessel[42])));
		if(Crew.AddCrewMember(&cAddUMmuToVessel[2], Age, 70, 70, &cAddUMmuToVessel[82]) == TRUE){
			sprintf_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "Crew \"%s\" aged %i added to the station", &cAddUMmuToVessel[2], Age);
		}
		else{
			strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "ERROR: Crew not added (station full?)");
		}
	}
}

void StationUCGO::UMMU_Init(FILEHANDLE cfg)
{
	// UMMU 3.0 Initialization
	char ExitPos_str[255];
	DOCKHANDLE dHandle;
	VECTOR3 dPos, dDir, dRot;

	Crew.InitUmmu(GetHandle());	
	float UMmuVersion = Crew.GetUserUMmuVersion();
	MaxEVASpots = 1;	// default to 1 and we'll find out if that changes during config file read

	if (!oapiReadItem_bool(cfg, "ShowDefaultHUD", ShowDefaultHUD)) {
		ShowDefaultHUD = FALSE;
	}

	if (!oapiReadItem_string(cfg, "ExitPosition1", ExitPos_str)) {
		// user did not specify an EVA location, so try to find a
		// main docking port and use its parameters to define the EVA point
		dHandle = GetDockHandle(0);		// only doing the main docking port for EVA
		if (dHandle != NULL) {
			GetDockParams(dHandle, dPos, dDir, dRot);
			x_pos[0] = dPos.x;
			z_pos[0] = dPos.z;
			Crew.DefineAirLockShape(FALSE, dPos.x-2,dPos.x+2,-2,2,dPos.z-2,dPos.z+2);
			Crew.SetMembersPosRotOnEVA(dPos, dRot);
		} else {	// not specified, and no docking port found, just give a default
			x_pos[0] = 0;
			z_pos[0] = 3;
			Crew.DefineAirLockShape(FALSE, -3, 3, -3, 3, -3, 3);
			Crew.SetMembersPosRotOnEVA(_V(0,0,3),_V(0,0,0));
		}
		Crew.DeclareActionArea(AIRLOCK_ACTION_AREA, _V(x_pos[0], 0, z_pos[0]), 2.0, TRUE,
			"action_door.wav", "Airlock doors activated.");
	}
	else {	// User specified at least one EVA location
		sscanf_s(ExitPos_str, "%f %f", &x_pos[0], &z_pos[0]);
		Crew.DefineAirLockShape(FALSE, x_pos[0] - 2, x_pos[0] + 2, -2, 2, z_pos[0] - 2, z_pos[0] + 2);
		Crew.SetMembersPosRotOnEVA(_V(x_pos[0], 0, z_pos[0]), _V(0, 0, 0));
		Crew.DeclareActionArea(0, _V(x_pos[0], 0, z_pos[0]), 2.0, TRUE,
			"action_door.wav", "Airlock doors activated.");
		if (oapiReadItem_string(cfg, "ExitPosition2", ExitPos_str)) {
			MaxEVASpots++;
			sscanf_s(ExitPos_str, "%f %f", &x_pos[1], &z_pos[1]);
			Crew.DefineAirLockShape(FALSE, x_pos[1] - 2, x_pos[1] + 2, -2, 2, z_pos[1] - 2, z_pos[1] + 2);
			Crew.SetMembersPosRotOnEVA(_V(x_pos[1], 0, z_pos[1]), _V(0, 0, 0));
			Crew.DeclareActionArea(1, _V(x_pos[1], 0, z_pos[1]), 2.0, TRUE,
				"action_door.wav", "Airlock doors activated.");
		}
		if (oapiReadItem_string(cfg, "ExitPosition3", ExitPos_str)) {
			MaxEVASpots++;
			sscanf_s(ExitPos_str, "%f %f", &x_pos[2], &z_pos[2]);
			Crew.DefineAirLockShape(FALSE, x_pos[2] - 2, x_pos[2] + 2, -2, 2, z_pos[2] - 2, z_pos[2] + 2);
			Crew.SetMembersPosRotOnEVA(_V(x_pos[2], 0, z_pos[2]), _V(0, 0, 0));
			Crew.DeclareActionArea(2, _V(x_pos[2], 0, z_pos[2]), 2.0, TRUE,
				"action_door.wav", "Airlock doors activated.");
		}
		if (oapiReadItem_string(cfg, "ExitPosition4", ExitPos_str)) {
			MaxEVASpots++;
			sscanf_s(ExitPos_str, "%f %f", &x_pos[3], &z_pos[3]);
			Crew.DefineAirLockShape(FALSE, x_pos[3] - 2, x_pos[3] + 2, -2, 2, z_pos[3] - 2, z_pos[3] + 2);
			Crew.SetMembersPosRotOnEVA(_V(x_pos[3], 0, z_pos[3]), _V(0, 0, 0));
			Crew.DeclareActionArea(3, _V(x_pos[3], 0, z_pos[3]), 2.0, TRUE,
				"action_door.wav", "Airlock doors activated.");
		}
	}

	for (int i = 0; i < MaxEVASpots; i++) {
		float mag = sqrt(x_pos[i] * x_pos[i] + z_pos[i] * z_pos[i]);
		CameraVector[i] = _V(x_pos[i] / mag, 0, z_pos[i] / mag);
	}

	if (!oapiReadItem_int(cfg, "SeatNumber", MaxUmmuSeats)) {
		MaxUmmuSeats = DEFAULT_MAX_CREW;
	}
	// UMMU 3.0 allows for maximum of 100 MMU slots per vessel	
	if (MaxUmmuSeats > 100) MaxUmmuSeats = 100;

	Crew.SetMaxSeatAvailableInShip(MaxUmmuSeats);

	cAddUMmuToVessel[0] = 0;
	SelectedUmmuMember = 0;  // our current selected member
	cUmmuHudDisplay[0] = 0;	// Initialisation of UMmu hud char variable
	dUMMUHudMessageDelay = 0;	// Initialisation of UMmu delay variable
	ActiveDockNumber = 0;	// Initilize initial selected dock to the first one defined
	SetUMMUAirlockPos();	// And set the airlock position for the initial active dock number
	CameraSwitchPending = false;
	strcpy_s(SendUMMUHudMessage(), sizeof(cUmmuHudDisplay), "UMMU is active.  Press '5' to see a list of UMMU command keys.");
}

void StationUCGO::UCGO_Init(FILEHANDLE cfg)
{
	char t_str[255];
	float x_pos, y_pos, z_pos;
	double slotinterval, speed;
	int cargorows, cargocols, dist;
	int row, col, slot;

	// UCGO 3.0 Initialization
	hUcgo.Init(GetHandle());

	if (!oapiReadItem_string(cfg, "CargoRowColumnNbr", t_str)) {
		cargorows = 1;
		cargocols = 1;
	} else {
		sscanf_s(t_str, "%d %d", &cargorows, &cargocols);
	}
	// UCGO allows for max 40 cargo slots per vessel
	if (cargorows * cargocols > 40) {
		cargorows = 8;
		cargocols = 5;
	}

	if (!oapiReadItem_string(cfg, "CargoSlotPos", t_str)) {
		// default placment of slot zero
		x_pos = 0;
		y_pos = 0;
		z_pos = 0;
	} else {
		sscanf_s(t_str, "%f %f %f", &x_pos, &y_pos, &z_pos);
	}

	if (!oapiReadItem_float(cfg, "CargoSlotInterval", slotinterval)) {
		slotinterval = 0.05;
	}

	slot = 0;
	for ( row = 1; row <= cargorows; row++) {
		for ( col = 1; col <= cargocols; col++) {
			hUcgo.DeclareCargoSlot(slot, _V(x_pos + (col-1)*slotinterval, y_pos, z_pos + (row-1)*slotinterval),_V(0,0,0));
			slot++;	
		}
	}

	if (!oapiReadItem_int(cfg, "CargoGrappleDist", dist)) {
		hUcgo.SetGrappleDistance(15);
	} else {
		hUcgo.SetGrappleDistance((dist < 2) ? 15 : dist);
	}

	if (oapiReadItem_string(cfg, "CargoReleasePos", t_str)) {
		sscanf_s(t_str, "%f %f", &x_pos, &z_pos);
		hUcgo.SetGlobalGroundReleasePos(_V(x_pos, 0, z_pos));
	}

	if (!oapiReadItem_float(cfg, "CargoMaxMass", MaxCargoMass)) {
		MaxCargoMass = 5000.0;
	}
	hUcgo.SetMaxCargoMassAcceptable(MaxCargoMass);

	hUcgo.SetSlotDoorState(TRUE);				// All cargo doors always open

	if (!oapiReadItem_float(cfg, "CargoReleaseSpeed", speed)) {
		hUcgo.SetReleaseSpeedInSpace(0.05f);
	} else {
		hUcgo.SetReleaseSpeedInSpace(speed);
	}

	// UCGO Variables initialisation
	cUCGOHudDisplay[0] = 0;
	dUCGOHudMessageDelay = 0;
	SelectedCargoSlot = 0;
	MaxCargoSlots = cargorows * cargocols;
	
	// welcome message with keys for users
	sprintf_s(SendUCGOHudMessage(), sizeof(cUCGOHudDisplay), "Welcome aboard the %s.  UCGO is active.  Press '6' to see a list of UCGO command keys.", GetName());
}

void StationUCGO::clbkVisualCreated(VISHANDLE vis, int refcount)
{
	hUcgo.SetUcgoVisual(vis);
}

// Allows user to set more than one egress/docking point per vessel
// Each time the user uses keys -/+ to change the EVA port, the active
// number is stored in ActiveDockNumber so that's all we need to
// properly set the current EVA dock point
void StationUCGO::SetUMMUAirlockPos(void)
{
	int AirlockStatus = Crew.GetAirlockDoorState();
	
	Crew.SetActiveDockForTransfer(ActiveDockNumber);

	Crew.DefineAirLockShape(AirlockStatus,
		x_pos[ActiveDockNumber] - 2, x_pos[ActiveDockNumber] + 2,
		-2.0, 2.0,
		z_pos[ActiveDockNumber] - 2, z_pos[ActiveDockNumber] + 2);
	
	Crew.SetMembersPosRotOnEVA(_V(x_pos[ActiveDockNumber], 0, z_pos[ActiveDockNumber]), _V(0, 0, 0));

	return;
}
